//===- ShardOps.h - Shard Dialect -------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#ifndef MLIR_DIALECT_SHARD_IR_SHARDDIALECT_H
#define MLIR_DIALECT_SHARD_IR_SHARDDIALECT_H

#include "mlir/IR/Dialect.h"

#include "mlir/Dialect/Shard/IR/ShardDialect.h.inc"

#endif // MLIR_DIALECT_SHARD_IR_SHARDDIALECT_H
